@@ -1,5 +1,5 @@
 /* defs.h -- data types and declarations.
-   Copyright (C) 1990, 91, 92, 93, 94, 2000, 2004, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1990, 91, 92, 93, 94, 2000, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,26 +19,37 @@
 #ifndef INC_DEFS_H
 #define INC_DEFS_H 1
 
-#include <config.h>
+#if !defined(ALREADY_INCLUDED_CONFIG_H)
+/*
+ * Savannah bug #20128: if we include some system header and it
+ * includes some othersecond system header, the second system header
+ * may in fact turn out to be a file provided by gnulib.  For that
+ * situation, we need to have already included <config.h> so that the
+ * Gnulib files have access to the information probed by their
+ * configure script fragments.  So <config.h> should be the first
+ * thing included.
+ */
+#error "<config.h> should be #included before defs.h, and indeed before any other header"
+Please stop compiling the program now
+#endif
+
+
 #include <sys/types.h>
 
 /* XXX: some of these includes probably don't belong in a common header file */
 #include <sys/stat.h>
-#include <stdio.h>
+#include <stdio.h>		/* for FILE* */
 #include <string.h>
-#include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
-#include <limits.h>
-#include <stdbool.h>
+#include <limits.h>		/* for CHAR_BIT */
+#include <stdbool.h>		/* for bool/boolean */
+#include <stdint.h>		/* for uintmax_t */
+#include <sys/stat.h> /* S_ISUID etc. */
 
 
 
-#ifndef errno
-extern int errno;
-#endif
-
 #ifndef CHAR_BIT
 # define CHAR_BIT 8
 #endif
@@ -52,13 +63,16 @@ typedef bool boolean;
 #include "timespec.h"
 #include "buildcmd.h"
 #include "quotearg.h"
-#include "stat_.h" /* S_ISUID etc. */
 
 /* These days we will assume ANSI/ISO C protootypes work on our compiler. */
 #define PARAMS(Args) Args
 
 #ifndef ATTRIBUTE_NORETURN
-# define ATTRIBUTE_NORETURN __attribute__ ((__noreturn__))
+# if HAVE_ATTRIBUTE_NORETURN
+#  define ATTRIBUTE_NORETURN __attribute__ ((__noreturn__))
+# else
+#  define ATTRIBUTE_NORETURN /* nothing */
+# endif
 #endif
 
 int optionl_stat PARAMS((const char *name, struct stat *p));
