@@ -1,6 +1,6 @@
 /* pred.c -- execute the expression tree.
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 2003, 
-                 2004, 2005 Free Software Foundation, Inc.
+                 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
    USA.
 */
 
+#include <config.h>
 #include "defs.h"
 
 #include <fnmatch.h>
@@ -27,6 +28,7 @@
 #include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include <assert.h>
 #include <stdarg.h>
 #include <fcntl.h>
@@ -594,22 +596,38 @@ pred_fprint0 (const char *pathname, struct stat *stat_buf, struct predicate *pre
 static char*
 mode_to_filetype(mode_t m)
 {
-  return
-    m == S_IFSOCK ? "s" :
-    m == S_IFLNK  ? "l" :
-    m == S_IFREG  ? "f" :
-    m == S_IFBLK  ? "b" :
-    m == S_IFDIR  ? "d" :
-    m == S_IFCHR  ? "c" :
-#ifdef S_IFDOOR
-    m == S_IFDOOR ? "D" :
+#define HANDLE_TYPE(t,letter) if (m==t) { return letter; }
+#ifdef S_IFREG
+  HANDLE_TYPE(S_IFREG,  "f");	/* regular file */
 #endif
-    m == S_IFIFO  ? "p" : "U";
+#ifdef S_IFDIR
+  HANDLE_TYPE(S_IFDIR,  "d");	/* directory */
+#endif
+#ifdef S_IFLNK
+  HANDLE_TYPE(S_IFLNK,  "l");	/* symbolic link */
+#endif
+#ifdef S_IFSOCK
+  HANDLE_TYPE(S_IFSOCK, "s");	/* Unix domain socket */
+#endif
+#ifdef S_IFBLK
+  HANDLE_TYPE(S_IFBLK,  "b");	/* block device */
+#endif
+#ifdef S_IFCHR
+  HANDLE_TYPE(S_IFCHR,  "c");	/* character device */
+#endif
+#ifdef S_IFIFO
+  HANDLE_TYPE(S_IFIFO,  "p");	/* FIFO */
+#endif
+#ifdef S_IFDOOR
+  HANDLE_TYPE(S_IFDOOR, "D");	/* Door (e.g. on Solaris) */
+#endif
+  return "U";			/* Unknown */
 }
 
 static double 
 file_sparseness(const struct stat *p)
 {
+#if defined(HAVE_STRUCT_STAT_ST_BLOCKS)
   if (0 == p->st_size)
     {
       if (0 == p->st_blocks)
@@ -622,6 +640,9 @@ file_sparseness(const struct stat *p)
       double blklen = file_blocksize(p) * (double)p->st_blocks;
       return blklen / p->st_size;
     }
+#else  
+  return 1.0;
+#endif
 }
 
 
@@ -2187,7 +2208,7 @@ print_list (FILE *fp, struct predicate *node)
   cur = node;
   while (cur != NULL)
     {
-      fprintf (fp, "%s ", blank_rtrim (cur->p_name, name));
+      fprintf (fp, "[%s] ", blank_rtrim (cur->p_name, name));
       cur = cur->pred_next;
     }
   fprintf (fp, "\n");
