@@ -61,6 +61,7 @@
 */
 
 #include <config.h>
+
 #include <stdio.h>
 #include <signal.h>
 #include <ctype.h>
@@ -72,6 +73,8 @@
 #include <getopt.h>
 #include <xstrtol.h>
 
+#include <stdbool.h>		/* for bool/boolean */
+
 /* The presence of unistd.h is assumed by gnulib these days, so we 
  * might as well assume it too. 
  */
@@ -92,11 +95,7 @@
 #include <stdlib.h>
 #endif
 
-#ifdef HAVE_ERRNO_H
 #include <errno.h>
-#else
-extern int errno;
-#endif
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -113,7 +112,7 @@ extern int errno;
 #ifdef gettext_noop
 # define N_(String) gettext_noop (String)
 #else
-/* We used to use (String) instead of just String, but apparentl;y ISO C
+/* We used to use (String) instead of just String, but apparently ISO C
  * doesn't allow this (at least, that's what HP said when someone reported
  * this as a compiler bug).  This is HP case number 1205608192.  See
  * also http://gcc.gnu.org/bugzilla/show_bug.cgi?id=11250 (which references 
@@ -125,9 +124,9 @@ extern int errno;
 
 #include "locatedb.h"
 #include <getline.h>
-#include "../gnulib/lib/xalloc.h"
-#include "../gnulib/lib/error.h"
-#include "../gnulib/lib/human.h"
+#include "xalloc.h"
+#include "error.h"
+#include "human.h"
 #include "dirname.h"
 #include "closeout.h"
 #include "nextelem.h"
@@ -347,6 +346,7 @@ struct process_data
   FILE *fp;			/* The pathname database.  */
   const char *dbfile;		/* Its name, or "<stdin>" */
   int  slocatedb_format;	/* Allows us to cope with slocate's format variant */
+  GetwordEndianState endian_state;
   /* for the old database format,
      the first and second characters of the most common bigrams.  */
   char bigram1[128];
@@ -369,49 +369,40 @@ static struct visitor *inspectors = NULL;
 static struct visitor *lastinspector = NULL;
 static struct visitor *past_pat_inspector = NULL;
 
-/* 0 or 1 pattern(s) */
-static int
-process_simple(struct process_data *procdata)
+static inline int visit(const struct visitor *p,
+			int accept_flags,
+			struct process_data *procdata,
+			const struct visitor * const stop)
 {
-  int result = VISIT_CONTINUE;
-  const struct visitor *p = inspectors;
-  
-  while ( ((VISIT_CONTINUE | VISIT_ACCEPTED) & result) && (NULL != p) )
+  register int result = accept_flags;
+  while ( (accept_flags & result) && (stop != p) )
     {
       result = (p->inspector)(procdata, p->context);
       p = p->next;
     }
+  return result;
+}
 
-    return result;
+/* 0 or 1 pattern(s) */
+static int
+process_simple(struct process_data *procdata)
+{
+  return visit(inspectors, (VISIT_CONTINUE|VISIT_ACCEPTED), procdata, NULL);
 }
 
 /* Accept if any pattern matches. */
 static int
 process_or (struct process_data *procdata)
 {
-  int result = VISIT_CONTINUE;
-  const struct visitor *p = inspectors;
+  int result;
   
-  while ( ((VISIT_CONTINUE | VISIT_REJECTED) & result) && (past_pat_inspector != p) )
-    {
-      result = (p->inspector)(procdata, p->context);
-      p = p->next;
-    }
-
+  result = visit(inspectors, (VISIT_CONTINUE|VISIT_REJECTED), procdata, past_pat_inspector);
   if (result == VISIT_CONTINUE)
     result = VISIT_REJECTED;
   if (result & (VISIT_ABORT | VISIT_REJECTED))
     return result;
 
-  p = past_pat_inspector;
-  result = VISIT_CONTINUE;
-
-  while ( (VISIT_CONTINUE == result) && (NULL != p) )
-    {
-      result = (p->inspector)(procdata, p->context);
-      p = p->next;
-    }
-  
+  result = visit(past_pat_inspector, VISIT_CONTINUE, procdata, NULL);
   if (VISIT_CONTINUE == result)
     return VISIT_ACCEPTED;
   else
@@ -422,29 +413,15 @@ process_or (struct process_data *procdata)
 static int
 process_and (struct process_data *procdata)
 {
-  int result = VISIT_CONTINUE;
-  const struct visitor *p = inspectors;
+  int result;
   
-  while ( ((VISIT_CONTINUE | VISIT_ACCEPTED) & result) && (past_pat_inspector != p) )
-    {
-      result = (p->inspector)(procdata, p->context);
-      p = p->next;
-    }
-
+  result = visit(inspectors, (VISIT_CONTINUE|VISIT_ACCEPTED), procdata, past_pat_inspector);
   if (result == VISIT_CONTINUE)
     result = VISIT_REJECTED;
   if (result & (VISIT_ABORT | VISIT_REJECTED))
     return result;
 
-  p = past_pat_inspector;
-  result = VISIT_CONTINUE;
-
-  while ( (VISIT_CONTINUE == result) && (NULL != p) )
-    {
-      result = (p->inspector)(procdata, p->context);
-      p = p->next;
-    }
-  
+  result = visit(past_pat_inspector, VISIT_CONTINUE, procdata, NULL);
   if (VISIT_CONTINUE == result)
     return VISIT_ACCEPTED;
   else
@@ -474,8 +451,6 @@ add_visitor(visitfunc fn, void *context)
     }
 }
 
-
-
 static int
 visit_justprint_quoted(struct process_data *procdata, void *context)
 {
@@ -496,38 +471,100 @@ visit_justprint_unquoted(struct process_data *procdata, void *context)
   return VISIT_CONTINUE;
 }
 
+static void
+toolong (struct process_data *procdata)
+{
+  error (1, 0,
+	 _("locate database %s contains a "
+	   "filename longer than locate can handle"),
+	 procdata->dbfile);
+}
+
+static void
+extend (struct process_data *procdata, size_t siz1, size_t siz2)
+{
+  /* Figure out if the addition operation is safe before performing it. */
+  if (SIZE_MAX - siz1 < siz2)
+    {
+      toolong (procdata);
+    }
+  else if (procdata->pathsize < (siz1+siz2))
+    {
+      procdata->pathsize = siz1+siz2;
+      procdata->original_filename = x2nrealloc (procdata->original_filename,
+						&procdata->pathsize,
+						1);
+    }
+}
+
 static int
 visit_old_format(struct process_data *procdata, void *context)
 {
-  register char *s;
+  register size_t i;
   (void) context;
 
+  if (EOF == procdata->c)
+    return VISIT_ABORT;
+
   /* Get the offset in the path where this path info starts.  */
   if (procdata->c == LOCATEDB_OLD_ESCAPE)
-    procdata->count += getw (procdata->fp) - LOCATEDB_OLD_OFFSET;
+    {
+      int minval, maxval;
+      int word;
+      
+      procdata->count -= LOCATEDB_OLD_OFFSET;
+      minval = (0       - procdata->count);
+      if (procdata->count >= 0)
+	maxval = (procdata->len - procdata->count);
+      else
+	maxval = (procdata->len - 0);
+      word = getword(procdata->fp, procdata->dbfile,
+		     minval, maxval, &procdata->endian_state);
+      procdata->count += word;
+      assert(procdata->count >= 0);
+    }
   else
-    procdata->count += procdata->c - LOCATEDB_OLD_OFFSET;
+    {
+      procdata->count += (procdata->c - LOCATEDB_OLD_OFFSET);
+      assert(procdata->count >= 0);
+    }
 
-  /* Overlay the old path with the remainder of the new.  */
-  for (s = procdata->original_filename + procdata->count;
+  /* Overlay the old path with the remainder of the new.  Read 
+   * more data until we get to the next filename.
+   */
+  for (i=procdata->count;
        (procdata->c = getc (procdata->fp)) > LOCATEDB_OLD_ESCAPE;)
-    if (procdata->c < 0200)
-      *s++ = procdata->c;		/* An ordinary character.  */
-    else
-      {
-	/* Bigram markers have the high bit set. */
-	procdata->c &= 0177;
-	*s++ = procdata->bigram1[procdata->c];
-	*s++ = procdata->bigram2[procdata->c];
-      }
-  *s-- = '\0';
+    {
+      if (EOF == procdata->c)
+	break;
 
+      if (procdata->c < 0200)
+	{
+	  /* An ordinary character. */	  
+	  extend (procdata, i, 1u);
+	  procdata->original_filename[i++] = procdata->c;
+	}
+      else
+	{
+	  /* Bigram markers have the high bit set. */
+	  extend (procdata, i, 2u);
+	  procdata->c &= 0177;
+	  procdata->original_filename[i++] = procdata->bigram1[procdata->c];
+	  procdata->original_filename[i++] = procdata->bigram2[procdata->c];
+	}
+    }
+
+  /* Consider the case where we executed the loop body zero times; we
+   * still need space for the terminating null byte. 
+   */
+  extend (procdata, i, 1u);
+  procdata->original_filename[i] = 0;
+  procdata->len = i;
   procdata->munged_filename = procdata->original_filename;
   
   return VISIT_CONTINUE;
 }
 
-
 static int
 visit_locate02_format(struct process_data *procdata, void *context)
 {
@@ -578,7 +615,8 @@ visit_locate02_format(struct process_data *procdata, void *context)
     }
  
   /* Overlay the old path with the remainder of the new.  */
-  nread = locate_read_str (&procdata->original_filename, &procdata->pathsize,
+  nread = locate_read_str (&procdata->original_filename,
+			   &procdata->pathsize,
 			   procdata->fp, 0, procdata->count);
   if (nread < 0)
     return VISIT_ABORT;
@@ -703,7 +741,7 @@ visit_non_existing_nofollow(struct process_data *procdata, void *context)
 }
 
 static int
-visit_substring_match_nocasefold(struct process_data *procdata, void *context)
+visit_substring_match_nocasefold_wide(struct process_data *procdata, void *context)
 {
   const char *pattern = context;
 
@@ -714,7 +752,18 @@ visit_substring_match_nocasefold(struct process_data *procdata, void *context)
 }
 
 static int
-visit_substring_match_casefold(struct process_data *procdata, void *context)
+visit_substring_match_nocasefold_narrow(struct process_data *procdata, void *context)
+{
+  const char *pattern = context;
+  assert(MB_CUR_MAX == 1);
+  if (NULL != strstr(procdata->munged_filename, pattern))
+    return VISIT_ACCEPTED;
+  else
+    return VISIT_REJECTED;
+}
+
+static int
+visit_substring_match_casefold_wide(struct process_data *procdata, void *context)
 {
   const char *pattern = context;
 
@@ -725,6 +774,19 @@ visit_substring_match_casefold(struct process_data *procdata, void *context)
 }
 
 
+static int
+visit_substring_match_casefold_narrow(struct process_data *procdata, void *context)
+{
+  const char *pattern = context;
+
+  assert(MB_CUR_MAX == 1);
+  if (NULL != strcasestr(procdata->munged_filename, pattern))
+    return VISIT_ACCEPTED;
+  else
+    return VISIT_REJECTED;
+}
+
+
 static int
 visit_globmatch_nofold(struct process_data *procdata, void *context)
 {
@@ -869,7 +931,13 @@ print_stats(int argc, size_t database_file_size)
 	{
 	  if (statistics.total_filename_length)
 	    {
-	      printf(_("Compression ratio %4.2f%%\n"),
+	      /* A negative compression ratio just means that the
+	       * compressed database is larger than the list of
+	       * filenames.  This can happen for example for
+	       * old-format databases containing a small list of short
+	       * filenames, because the bigram list is 256 bytes.
+	       */
+	      printf(_("Compression ratio %4.2f%% (higher is better)\n"),
 		     100.0 * ((double)statistics.total_filename_length
 			      - (double) database_file_size)
 		     / (double) statistics.total_filename_length);
@@ -957,6 +1025,24 @@ looking_at_slocate_locatedb (const char *filename,
     }
 }
 
+
+static int 
+i_am_little_endian(void)
+{
+  union 
+  {
+    unsigned char uch[4];
+    unsigned int ui;
+  } u;
+  u.ui = 0u;
+  u.uch[0] = 1;
+  u.uch[1] = u.uch[2] = u.uch[3] = 0;
+  return u.ui == 1;
+}
+
+
+
+
 /* Print or count the entries in DBFILE that match shell globbing patterns in 
    ARGV. Return the number of entries matched. */
 
@@ -981,6 +1067,7 @@ search_one_database (int argc,
   int nread;		     /* number of bytes read from an entry. */
   struct process_data procdata;	/* Storage for data shared with visitors. */
   int slocate_seclevel;
+  int oldformat;
   struct visitor* pvis; /* temp for determining past_pat_inspector. */
   const char *format_name;
   enum ExistenceCheckType do_check_existence;
@@ -997,7 +1084,9 @@ search_one_database (int argc,
   
   if (ignore_case)
     regex_options |= RE_ICASE;
-  
+
+  oldformat = 0;
+  procdata.endian_state = GetwordEndianStateInitial;
   procdata.len = procdata.count = 0;
   procdata.slocatedb_format = 0;
   procdata.itemcount = 0;
@@ -1010,8 +1099,11 @@ search_one_database (int argc,
   lastinspector = NULL;
   past_pat_inspector = NULL;
   results_were_filtered = false;
-  
+#if 0  
   procdata.pathsize = 1026;	/* Increased as necessary by locate_read_str.  */
+#else
+  procdata.pathsize = 128;	/* Increased as necessary by locate_read_str.  */
+#endif
   procdata.original_filename = xmalloc (procdata.pathsize);
 
 
@@ -1082,6 +1174,7 @@ search_one_database (int argc,
       int nread2;
       
       procdata.slocatedb_format = 0;
+      extend (&procdata, sizeof(LOCATEDB_MAGIC), 0u);
       nread2 = fread (procdata.original_filename+nread, 1, sizeof (LOCATEDB_MAGIC)-nread,
 		      procdata.fp);
       if (looking_at_gnu_locatedb(procdata.original_filename, nread+nread2))
@@ -1097,6 +1190,7 @@ search_one_database (int argc,
 	  /* Read the list of the most common bigrams in the database.  */
 	  if (nread < 256)
 	    {
+	      extend (&procdata, 256u, 0u);
 	      int more_read = fread (procdata.original_filename + nread, 1,
 				     256 - nread, procdata.fp);
 	      if ( (more_read + nread) != 256 )
@@ -1115,6 +1209,7 @@ search_one_database (int argc,
 	      procdata.bigram2[i] = procdata.original_filename[(i << 1) + 1];
 	    }
 	  format_name = "old";
+	  oldformat = 1;
 	  add_visitor(visit_old_format, NULL);
 	}
     }
@@ -1167,10 +1262,26 @@ search_one_database (int argc,
 	   * traditional behaviour.
 	   * James Youngman <jay@gnu.org> 
 	   */
-	  if (ignore_case)
-	    add_visitor(visit_substring_match_casefold, pathpart);
+	  visitfunc matcher;
+	  if (1 == MB_CUR_MAX)
+	    {
+	      /* As an optimisation, use a strstr() matcher if we are
+	       * in a unibyte locale.  This can give a x2 speedup in
+	       * the C locale.  Some light testing reveals that
+	       * glibc's strstr() is somewhere around 40% faster than
+	       * gnulib's, so we just use strstr().
+	       */
+	      matcher = ignore_case ?
+		visit_substring_match_casefold_narrow  :
+		visit_substring_match_nocasefold_narrow;
+	    }
 	  else
-	    add_visitor(visit_substring_match_nocasefold, pathpart);
+	    {
+	      matcher = ignore_case ?
+		visit_substring_match_casefold_wide  :
+		visit_substring_match_nocasefold_wide;
+	    }
+	  add_visitor(matcher, pathpart);
 	}
     }
 
@@ -1237,9 +1348,9 @@ search_one_database (int argc,
 
   if (stats)
     {
-	printf(_("Database %s is in the %s format.\n"),
-	       procdata.dbfile,
-	       format_name);
+      printf(_("Database %s is in the %s format.\n"),
+	     procdata.dbfile,
+	     format_name);
     }
 
 
@@ -1255,6 +1366,28 @@ search_one_database (int argc,
   
   if (stats)
     {
+      if (oldformat)
+	{
+	  int host_little_endian = i_am_little_endian();
+	  const char *little = _("The database has little-endian "
+				 "machine-word encoding.\n");
+	  const char *big    = _("The database has big-endian "
+				 "machine-word encoding.\n");
+	  
+	  if (GetwordEndianStateNative == procdata.endian_state)
+	    {
+	      printf("%s", (host_little_endian ? little : big));
+	    }
+	  else if (GetwordEndianStateSwab == procdata.endian_state)
+	    {
+	      printf("%s", (host_little_endian ? big : little));
+	    }
+	  else
+	    {
+	      printf(_("The database machine-word encoding order "
+		       "is not obvious.\n"));
+	    }
+	}
       if (filesize)
 	print_stats(argc, filesize);
     }
@@ -1332,7 +1465,8 @@ drop_privs(void)
   const uid_t orig_euid = geteuid();
   const uid_t uid       = getuid();
   const gid_t gid       = getgid();
-  
+
+#if HAVE_SETGROUPS  
   /* Use of setgroups() is restricted to root only. */
   if (0 == orig_euid)
     {
@@ -1345,6 +1479,7 @@ drop_privs(void)
 	  goto fail;
 	}
     }
+#endif
 
   /* Drop any setuid privileges */
   if (uid != orig_euid)
