@@ -62,6 +62,20 @@
 #  endif
 # endif
 
+typedef enum 
+  { 
+    GetwordEndianStateInitial = 0,
+    GetwordEndianStateNative  = 1,
+    GetwordEndianStateSwab    = 2
+  } GetwordEndianState;
+
+int getword (FILE *fp, const char *filename,
+	     size_t minvalue, size_t maxvalue,
+	     GetwordEndianState *endian_state_flag);
+
+bool putword (FILE *fp, int word,
+	      GetwordEndianState endian_state_flag);
+
 
 #define SLOCATE_DB_MAGIC_LEN 2
 
